@@ -19,9 +19,10 @@ OUTDIR=$(mktemp -d)
 TAG_NAME="v$1"
 
 cd $DIR
+python3 misc/update_version.py "$1"
 
 echo ">>>>> Checking changelog"
-grep -A 5 -F $1 CHANGELOG.md || true
+grep -A 5 -F "$1" CHANGELOG.md || true
 prompt "Is the changelog correct and complete?"
 
 echo ">>>>> Checking Doxyfile"
@@ -49,20 +50,19 @@ cp -r html libcbor_docs_html
 tar -zcf libcbor_docs.tar.gz libcbor_docs_html
 
 cp -r doxygen/html libcbor_api_docs_html
-cp -r doxygen/html $DIR/docs/doxygen
 tar -zcf libcbor_api_docs.tar.gz libcbor_api_docs_html
 
-mv libcbor_docs.tar.gz libcbor_api_docs.tar.gz $OUTDIR
+mv libcbor_docs.tar.gz libcbor_api_docs.tar.gz "$OUTDIR"
 
-cd $OUTDIR
-
-cmake $DIR -DCMAKE_BUILD_TYPE=Release -DWITH_TESTS=ON
+pushd "$OUTDIR"
+cmake "$DIR" -DCMAKE_BUILD_TYPE=Release -DWITH_TESTS=ON
 make
 ctest
+popd
 
 prompt "Will proceed to tag the release with $TAG_NAME."
-git tag $TAG_NAME
-git push origin
+git tag "$TAG_NAME"
+git push --tags
 
 set +x
 
@@ -71,4 +71,5 @@ echo "Add the release to GitHub at https://github.com/PJK/libcbor/releases/new *
 prompt "Have you added the release to https://github.com/PJK/libcbor/releases/tag/$TAG_NAME?"
 
 echo "Update the Hombrew formula (https://github.com/Homebrew/homebrew-core/blob/master/Formula/libcbor.rb) *now*"
-prompt "Have you updated the formula?"
\ No newline at end of file
+echo "HOWTO: https://github.com/Linuxbrew/brew/blob/master/docs/How-To-Open-a-Homebrew-Pull-Request.md"
+prompt "Have you updated the formula?"
