@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 Pavel Kalvoda <me@pavelkalvoda.com>
+ * Copyright (c) 2014-2020 Pavel Kalvoda <me@pavelkalvoda.com>
  *
  * libcbor is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -82,8 +82,6 @@ cbor_item_t *cbor_load(cbor_data source, size_t source_size,
           result->error.code = CBOR_ERR_NOTENOUGHDATA;
           goto error;
         }
-      case CBOR_DECODER_EBUFFER:
-        /* Fallthrough */
       case CBOR_DECODER_ERROR:
         /* Reserved/malformated item */
         {
@@ -213,8 +211,9 @@ cbor_item_t *cbor_copy(cbor_item_t *item) {
       return res;
     }
     case CBOR_TYPE_TAG:
-      return cbor_build_tag(cbor_tag_value(item),
-                            cbor_move(cbor_copy(cbor_tag_item(item))));
+      return cbor_build_tag(
+          cbor_tag_value(item),
+          cbor_move(cbor_copy(cbor_move(cbor_tag_item(item)))));
     case CBOR_TYPE_FLOAT_CTRL:
       return _cbor_copy_float_ctrl(item);
   }
@@ -316,15 +315,14 @@ static void _cbor_nested_describe(cbor_item_t *item, FILE *out, int indent) {
     case CBOR_TYPE_TAG: {
       fprintf(out, "%*s[CBOR_TYPE_TAG] ", indent, " ");
       fprintf(out, "Value: %" PRIu64 "\n", cbor_tag_value(item));
-      _cbor_nested_describe(cbor_tag_item(item), out, indent + 4);
+      _cbor_nested_describe(cbor_move(cbor_tag_item(item)), out, indent + 4);
       break;
     };
     case CBOR_TYPE_FLOAT_CTRL: {
       fprintf(out, "%*s[CBOR_TYPE_FLOAT_CTRL] ", indent, " ");
       if (cbor_float_ctrl_is_ctrl(item)) {
         if (cbor_is_bool(item))
-          fprintf(out, "Bool: %s\n",
-                  cbor_ctrl_is_bool(item) ? "true" : "false");
+          fprintf(out, "Bool: %s\n", cbor_get_bool(item) ? "true" : "false");
         else if (cbor_is_undef(item))
           fprintf(out, "Undefined\n");
         else if (cbor_is_null(item))
