@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 Pavel Kalvoda <me@pavelkalvoda.com>
+ * Copyright (c) 2014-2020 Pavel Kalvoda <me@pavelkalvoda.com>
  *
  * libcbor is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -256,8 +256,8 @@ size_t cbor_serialize_tag(const cbor_item_t *item, unsigned char *buffer,
   size_t written = cbor_encode_tag(cbor_tag_value(item), buffer, buffer_size);
   if (written == 0) return 0;
 
-  size_t item_written = cbor_serialize(cbor_tag_item(item), buffer + written,
-                                       buffer_size - written);
+  size_t item_written = cbor_serialize(cbor_move(cbor_tag_item(item)),
+                                       buffer + written, buffer_size - written);
   if (item_written == 0)
     return 0;
   else
