@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 Pavel Kalvoda <me@pavelkalvoda.com>
+ * Copyright (c) 2014-2020 Pavel Kalvoda <me@pavelkalvoda.com>
  *
  * libcbor is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -8,13 +8,12 @@
 #include "streaming.h"
 #include "internal/loaders.h"
 
-bool static _cbor_claim_bytes(size_t required, size_t provided,
-                              struct cbor_decoder_result *result) {
+bool static claim_bytes(size_t required, size_t provided,
+                        struct cbor_decoder_result *result) {
   if (required > (provided - result->read)) {
-    /* We need to keep all the metadata if parsing is to be resumed */
+    result->required = required + result->read;
     result->read = 0;
     result->status = CBOR_DECODER_NEDATA;
-    result->required = required;
     return false;
   } else {
     result->read += required;
@@ -26,14 +25,12 @@ bool static _cbor_claim_bytes(size_t required, size_t provided,
 struct cbor_decoder_result cbor_stream_decode(
     cbor_data source, size_t source_size,
     const struct cbor_callbacks *callbacks, void *context) {
-  /* If we have no data, we cannot read even the MTB */
-  if (source_size < 1) {
-    return (struct cbor_decoder_result){0, CBOR_DECODER_EBUFFER};
+  // Attempt to claim the initial MTB byte
+  struct cbor_decoder_result result = {.status = CBOR_DECODER_FINISHED};
+  if (!claim_bytes(1, source_size, &result)) {
+    return result;
   }
 
-  /* If we have a byte, assume it's the MTB */
-  struct cbor_decoder_result result = {1, CBOR_DECODER_FINISHED};
-
   switch (*source) {
     case 0x00: /* Fallthrough */
     case 0x01: /* Fallthrough */
@@ -67,7 +64,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x18:
       /* One byte unsigned integer */
       {
-        if (_cbor_claim_bytes(1, source_size, &result)) {
+        if (claim_bytes(1, source_size, &result)) {
           callbacks->uint8(context, _cbor_load_uint8(source + 1));
         }
         return result;
@@ -75,7 +72,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x19:
       /* Two bytes unsigned integer */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           callbacks->uint16(context, _cbor_load_uint16(source + 1));
         }
         return result;
@@ -83,7 +80,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x1A:
       /* Four bytes unsigned integer */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           callbacks->uint32(context, _cbor_load_uint32(source + 1));
         }
         return result;
@@ -91,7 +88,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x1B:
       /* Eight bytes unsigned integer */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           callbacks->uint64(context, _cbor_load_uint64(source + 1));
         }
         return result;
@@ -135,7 +132,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x38:
       /* One byte negative integer */
       {
-        if (_cbor_claim_bytes(1, source_size, &result)) {
+        if (claim_bytes(1, source_size, &result)) {
           callbacks->negint8(context, _cbor_load_uint8(source + 1));
         }
         return result;
@@ -143,7 +140,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x39:
       /* Two bytes negative integer */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           callbacks->negint16(context, _cbor_load_uint16(source + 1));
         }
         return result;
@@ -151,7 +148,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x3A:
       /* Four bytes negative integer */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           callbacks->negint32(context, _cbor_load_uint32(source + 1));
         }
         return result;
@@ -159,7 +156,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x3B:
       /* Eight bytes negative integer */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           callbacks->negint64(context, _cbor_load_uint64(source + 1));
         }
         return result;
@@ -198,7 +195,7 @@ struct cbor_decoder_result cbor_stream_decode(
       {
         size_t length =
             (size_t)_cbor_load_uint8(source) - 0x40; /* 0x40 offset */
-        if (_cbor_claim_bytes(length, source_size, &result)) {
+        if (claim_bytes(length, source_size, &result)) {
           callbacks->byte_string(context, source + 1, length);
         }
         return result;
@@ -207,9 +204,9 @@ struct cbor_decoder_result cbor_stream_decode(
       /* One byte length byte string */
       // TODO template this?
       {
-        if (_cbor_claim_bytes(1, source_size, &result)) {
+        if (claim_bytes(1, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint8(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->byte_string(context, source + 1 + 1, length);
           }
         }
@@ -218,9 +215,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x59:
       /* Two bytes length byte string */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint16(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->byte_string(context, source + 1 + 2, length);
           }
         }
@@ -229,9 +226,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x5A:
       /* Four bytes length byte string */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint32(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->byte_string(context, source + 1 + 4, length);
           }
         }
@@ -240,9 +237,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x5B:
       /* Eight bytes length byte string */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint64(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->byte_string(context, source + 1 + 8, length);
           }
         }
@@ -287,7 +284,7 @@ struct cbor_decoder_result cbor_stream_decode(
       {
         size_t length =
             (size_t)_cbor_load_uint8(source) - 0x60; /* 0x60 offset */
-        if (_cbor_claim_bytes(length, source_size, &result)) {
+        if (claim_bytes(length, source_size, &result)) {
           callbacks->string(context, source + 1, length);
         }
         return result;
@@ -295,9 +292,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x78:
       /* One byte length string */
       {
-        if (_cbor_claim_bytes(1, source_size, &result)) {
+        if (claim_bytes(1, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint8(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->string(context, source + 1 + 1, length);
           }
         }
@@ -306,9 +303,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x79:
       /* Two bytes length string */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint16(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->string(context, source + 1 + 2, length);
           }
         }
@@ -317,9 +314,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x7A:
       /* Four bytes length string */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint32(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->string(context, source + 1 + 4, length);
           }
         }
@@ -328,9 +325,9 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x7B:
       /* Eight bytes length string */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           size_t length = (size_t)_cbor_load_uint64(source + 1);
-          if (_cbor_claim_bytes(length, source_size, &result)) {
+          if (claim_bytes(length, source_size, &result)) {
             callbacks->string(context, source + 1 + 8, length);
           }
         }
@@ -380,7 +377,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x98:
       /* One byte length array */
       {
-        if (_cbor_claim_bytes(1, source_size, &result)) {
+        if (claim_bytes(1, source_size, &result)) {
           callbacks->array_start(context, (size_t)_cbor_load_uint8(source + 1));
         }
         return result;
@@ -388,7 +385,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x99:
       /* Two bytes length array */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           callbacks->array_start(context,
                                  (size_t)_cbor_load_uint16(source + 1));
         }
@@ -397,7 +394,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x9A:
       /* Four bytes length array */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           callbacks->array_start(context,
                                  (size_t)_cbor_load_uint32(source + 1));
         }
@@ -406,7 +403,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x9B:
       /* Eight bytes length array */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           callbacks->array_start(context,
                                  (size_t)_cbor_load_uint64(source + 1));
         }
@@ -456,7 +453,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xB8:
       /* One byte length map */
       {
-        if (_cbor_claim_bytes(1, source_size, &result)) {
+        if (claim_bytes(1, source_size, &result)) {
           callbacks->map_start(context, (size_t)_cbor_load_uint8(source + 1));
         }
         return result;
@@ -464,7 +461,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xB9:
       /* Two bytes length map */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           callbacks->map_start(context, (size_t)_cbor_load_uint16(source + 1));
         }
         return result;
@@ -472,7 +469,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xBA:
       /* Four bytes length map */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           callbacks->map_start(context, (size_t)_cbor_load_uint32(source + 1));
         }
         return result;
@@ -480,7 +477,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xBB:
       /* Eight bytes length map */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           callbacks->map_start(context, (size_t)_cbor_load_uint64(source + 1));
         }
         return result;
@@ -541,28 +538,28 @@ struct cbor_decoder_result cbor_stream_decode(
     }
     case 0xD8: /* 1B tag */
     {
-      if (_cbor_claim_bytes(1, source_size, &result)) {
+      if (claim_bytes(1, source_size, &result)) {
         callbacks->tag(context, _cbor_load_uint8(source + 1));
       }
       return result;
     }
     case 0xD9: /* 2B tag */
     {
-      if (_cbor_claim_bytes(2, source_size, &result)) {
+      if (claim_bytes(2, source_size, &result)) {
         callbacks->tag(context, _cbor_load_uint16(source + 1));
       }
       return result;
     }
     case 0xDA: /* 4B tag */
     {
-      if (_cbor_claim_bytes(4, source_size, &result)) {
+      if (claim_bytes(4, source_size, &result)) {
         callbacks->tag(context, _cbor_load_uint32(source + 1));
       }
       return result;
     }
     case 0xDB: /* 8B tag */
     {
-      if (_cbor_claim_bytes(8, source_size, &result)) {
+      if (claim_bytes(8, source_size, &result)) {
         callbacks->tag(context, _cbor_load_uint64(source + 1));
       }
       return result;
@@ -627,7 +624,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xF9:
       /* 2B float */
       {
-        if (_cbor_claim_bytes(2, source_size, &result)) {
+        if (claim_bytes(2, source_size, &result)) {
           callbacks->float2(context, _cbor_load_half(source + 1));
         }
         return result;
@@ -635,7 +632,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xFA:
       /* 4B float */
       {
-        if (_cbor_claim_bytes(4, source_size, &result)) {
+        if (claim_bytes(4, source_size, &result)) {
           callbacks->float4(context, _cbor_load_float(source + 1));
         }
         return result;
@@ -643,7 +640,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xFB:
       /* 8B float */
       {
-        if (_cbor_claim_bytes(8, source_size, &result)) {
+        if (claim_bytes(8, source_size, &result)) {
           callbacks->float8(context, _cbor_load_double(source + 1));
         }
         return result;
