@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 Pavel Kalvoda <me@pavelkalvoda.com>
+ * Copyright (c) 2014-2020 Pavel Kalvoda <me@pavelkalvoda.com>
  *
  * libcbor is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -22,7 +22,7 @@ cbor_item_t *cbor_new_tag(uint64_t value) {
 
 cbor_item_t *cbor_tag_item(const cbor_item_t *item) {
   assert(cbor_isa_tag(item));
-  return item->metadata.tag_metadata.tagged_item;
+  return cbor_incref(item->metadata.tag_metadata.tagged_item);
 }
 
 uint64_t cbor_tag_value(const cbor_item_t *item) {
